﻿<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"    
    xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
    xmlns:x="http://schemas.microsoft.com/xps/2005/06/resourcedictionary-key">
  
    <xsl:output method="xml" indent="yes"/>
    <xsl:include href="../include/locale.xslt"/>
    <xsl:include href="../include/styles.xslt"/>
    <xsl:include href="../include/deliveryHelpers.xslt"/>

  
    <xsl:template match="/ReportData">
      <FlowDocument        
        PageWidth="21cm"
        PageHeight="29.7cm"      
        PagePadding="60,40,40,40"      
        LineHeight="25"
        ColumnWidth="21 cm">
        
        <xsl:call-template name="styles"/>


        <xsl:call-template name="reportHeader">
          <xsl:with-param name="title" select="$_PROJECT_SCHEDULE"/>
        </xsl:call-template>

        <xsl:variable name="deliveriedItems" select="Project/Sprints/Sprint[SprintNumber &lt; /ReportData/ProjectCurrentSprintNumber]"/>

        <Paragraph Style="{{StaticResource GroupParagraph}}">
          <xsl:value-of select="$_Project_start"/>
          <xsl:call-template name="formatDate">
            <xsl:with-param name="dateTime" select="Project/Sprints/Sprint[SprintNumber = 1]/StartDate" />
          </xsl:call-template>
        </Paragraph>


        <xsl:if test="count($deliveriedItems) &gt; 0">
          <Paragraph Style="{{StaticResource GroupParagraph}}">
            <xsl:value-of select="$_DELIVERED_ITEMS"/>
          </Paragraph>
          <Table>
            <TableRowGroup>
              <xsl:for-each select="$deliveriedItems">
                <xsl:sort select="SprintNumber" data-type="number"/>
                <TableRow>
                  <TableCell>
                    <Paragraph FontSize="16" FontWeight="Bold" Margin="00,15,0,0" TextAlignment="Center" Foreground="Gray">
                      Sprint <xsl:value-of select="SprintNumber"/>
                    </Paragraph>
                    <xsl:call-template name="sprintDeliveries">
                      <xsl:with-param name="sprintNumber" select="SprintNumber"/>
                    </xsl:call-template>
                  </TableCell>
                </TableRow>
              </xsl:for-each>
            </TableRowGroup>
          </Table>

        </xsl:if>
        


        <Paragraph Style="{{StaticResource GroupParagraph}}">
          <xsl:value-of select="$_SCHEDULED_ITEMS"/>          
        </Paragraph>
        <Table>
          <TableRowGroup>
            <xsl:for-each select="Project/Sprints/Sprint[SprintNumber &gt;= /ReportData/ProjectCurrentSprintNumber]">
              <xsl:sort select="SprintNumber" data-type="number"/>
              <TableRow>
                <TableCell>
                  <Paragraph FontSize="16" FontWeight="Bold" Margin="00,15,0,0" TextAlignment="Center" Foreground="Gray">
                    Sprint <xsl:value-of select="SprintNumber"/>                    
                  </Paragraph>
                  <xsl:call-template name="sprintDeliveries">
                    <xsl:with-param name="sprintNumber" select="SprintNumber"/>
                  </xsl:call-template>
                </TableCell>
              </TableRow>
            </xsl:for-each>            
          </TableRowGroup>  
        </Table>

        <Table CellSpacing="4" Margin="00,60,0,0">
          <Table.Columns>
            <TableColumn  />
            <TableColumn Width="20" />
            <TableColumn Width="300" />
          </Table.Columns>
          <TableRowGroup>
            <TableRow>
              <TableCell/>
              <TableCell Style="{{StaticResource deliveryItemCell}}">
                <Paragraph>
                </Paragraph>
              </TableCell>
              <TableCell>
                <Paragraph FontSize="10">
                  <xsl:value-of select="$_Delivery_item"/>                  
                </Paragraph>
              </TableCell>
            </TableRow>
            <TableRow>
              <TableCell/>
              <TableCell Style="{{StaticResource criticalItemCell}}">
                <Paragraph>
                </Paragraph>
              </TableCell>
              <TableCell>
                <Paragraph  FontSize="10">
                  <xsl:value-of select="$_Critical_item"/>
                </Paragraph>
              </TableCell>
            </TableRow>
          </TableRowGroup>
        </Table>

        
        
        
      </FlowDocument>
    </xsl:template>
  


</xsl:stylesheet>
