﻿<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
                xmlns:exslt="http://exslt.org/common"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation">
    <xsl:output method="xml" indent="yes"/>

  <xsl:include href="helpers.xslt"/>


  <xsl:template name="_sprintItems">
    <xsl:param name="sprintItems"/>
    <xsl:param name="usePreviousPlan"/>
    <xsl:param name="hideDeliveryDate" select="0"/>
    <xsl:param name="endDate"/>    
    <xsl:for-each select="$sprintItems/BacklogItem">
      <xsl:sort select="OccurrenceConstraint" data-type="number"/>
      <xsl:sort select="BusinessPriority" data-type="number"/>
      <xsl:variable name="itemStyle">
        <xsl:choose>
          <xsl:when test="IssueType = 3 and $hideDeliveryDate = 0" >{StaticResource criticalItemCell}</xsl:when>
          <xsl:when test="(OccurrenceConstraint = 2 or string-length(DeliveryDate) &gt; 0) and $hideDeliveryDate = 0" >{StaticResource deliveryItemCell}</xsl:when>          
          <xsl:otherwise>{StaticResource normalItemCell}</xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      
      <TableRow >
        <xsl:if test="$hideDeliveryDate = 0">
          <TableCell Style="{$itemStyle}" TextAlignment="Center">

            <Paragraph FontWeight="Bold" Foreground="White">

              <xsl:if test="string-length(DeliveryDate) &gt; 0">
                <xsl:call-template name="formatShortDate">
                  <xsl:with-param name="dateTime" select="DeliveryDate" />
                </xsl:call-template>
              </xsl:if>
              
              <xsl:if test="string-length(DeliveryDate) = 0 and (OccurrenceConstraint = 2 or IssueType = 3)">
                <xsl:call-template name="formatShortDate">
                  <xsl:with-param name="dateTime" select="$endDate" />
                </xsl:call-template>
              </xsl:if>
              
              
              
              
            </Paragraph>


          </TableCell>
        </xsl:if>        
        <TableCell Style="{{StaticResource normalItemCell}}">
          <Paragraph>
            <xsl:value-of select="Name"/>
            <TextBlock FontSize="10" Foreground="Gray" Margin="3,0,0,0" VerticalAlignment="Top">
              [<xsl:value-of select="BacklogItemNumber"/>]
            </TextBlock>
          </Paragraph>
        </TableCell>
        <TableCell Style="{{StaticResource normalItemCell}}" TextAlignment="Right">
          <Paragraph>
            <xsl:variable name="totalHours">
              <xsl:choose>
                <xsl:when test="$usePreviousPlan">
                  <xsl:value-of select="sum(PreviousPlannedHours/PlannedHour/Hours)" />
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="sum(CurrentPlannedHours/PlannedHour/Hours)" />
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            <xsl:value-of select="$totalHours"/> hrs
          </Paragraph>
        </TableCell>
        <TableCell Style="{{StaticResource normalItemCell}}" TextAlignment="Center">
          <Paragraph>
            <xsl:call-template name="itemStatus" />
          </Paragraph>
        </TableCell>
    
      </TableRow>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="sprintDeliveries">    
    <xsl:param name="sprintNumber"/>
    <xsl:param name="usePreviousPlan"/>
    <xsl:param name="hideDeliveryDate" select="0"/>
    <Table>
      <Table.Columns>
        <xsl:if test="$hideDeliveryDate = 0">
          <TableColumn Width="80" />
        </xsl:if>        
        <TableColumn />
        <TableColumn Width="60" />
        <TableColumn Width="100" />        
      </Table.Columns>
      <TableRowGroup>
        
        <xsl:variable name="sprintItems">
          <xsl:choose>
            <xsl:when test="$usePreviousPlan">
              <xsl:copy-of select="/ReportData/ArrayOfBacklogItem/BacklogItem[PreviousPlannedHours/PlannedHour/SprintNumber = $sprintNumber]"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:copy-of select="/ReportData/ArrayOfBacklogItem/BacklogItem[SprintNumber = $sprintNumber]"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>

        <xsl:call-template name="_sprintItems">
          <xsl:with-param name="sprintItems" select="exslt:node-set($sprintItems)"/>
          <xsl:with-param name="usePreviousPlan" select="$usePreviousPlan"/>
          <xsl:with-param name="hideDeliveryDate" select="$hideDeliveryDate"/>
          <xsl:with-param name="endDate" select="/ReportData/Project/Sprints/Sprint[SprintNumber = $sprintNumber]/EndDate"/>          
        </xsl:call-template>
     
      </TableRowGroup>
    </Table>
  </xsl:template>



</xsl:stylesheet>
