﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using ScrumFactory.Composition.ViewModel;
using ScrumFactory.Composition;
using ScrumFactory.Windows.Helpers;
using ScrumFactory.Services;
using System.Linq;
using ScrumFactory.Composition.View;
using System.Windows.Data;
using System.Windows.Input;

namespace ScrumFactory.Team.ViewModel {
    
    [Export]
    [Export(typeof(ITopMenuViewModel))]
    public class MyDayViewModel : MemberDetailBase, ITopMenuViewModel, INotifyPropertyChanged {

        private ICollection<ProjectEvent> projectEvents;

        private bool isTeamTasksVisible = false;

        public ICollection<ProjectInfo> RecentProjects { get; private set; }

        [ImportingConstructor]
        public MyDayViewModel(
            [Import] IEventAggregator aggregator,
            [Import] ITasksService tasksService,
            [Import] IProjectsService projectsService,
            [Import] ITeamService teamService,
            [Import] IBackgroundExecutor executor,
            [Import] IAuthorizationService authorizator) : base(aggregator, tasksService, projectsService, teamService, executor, authorizator)  {

            aggregator.Subscribe<MemberProfile>(ScrumFactoryEvent.SignedMemberChanged, OnSignedMemberChanged);

            aggregator.Subscribe<Task>(ScrumFactoryEvent.TaskAdded, t => { NeedRefresh = true; });
            aggregator.Subscribe<Task>(ScrumFactoryEvent.TaskChanged, t => { NeedRefresh = true; });

            aggregator.Subscribe<ICollection<ProjectInfo>>(ScrumFactoryEvent.RecentProjectChanged, prjs => {
                RecentProjects = prjs.Take(8).ToArray();
                OnPropertyChanged("RecentProjects");
            });
            
            OnLoadCommand = new DelegateCommand(OnLoad);
            RefreshCommand = new DelegateCommand(Load);
            ShowMemberDetailCommand = new DelegateCommand<MemberProfile>(ShowMemberDetail);
         
        }


        public bool IsTeamTasksVisible {
            get {
                return isTeamTasksVisible;
            }
            set {
                if(!isTeamTasksVisible && (TeamMembers==null || TeamMembers.Count==0) && value)
                    LoadMemberTeamTasks();
                isTeamTasksVisible = value;                
                OnPropertyChanged("IsTeamTasksVisible");
            }
        }

        [Import]
        public IPendingMembershipsListViewModel PendingMembershipsListViewModel { get; set; }

        [Import]
        private MemberDetailViewModel MemberDetailViewModel  { get; set; }

        private void ShowMemberDetail(MemberProfile member) {
            if (member == null || member.MemberUId==null)
                return;
            MemberDetailViewModel.Show(this, member);
        }

        private void OnSignedMemberChanged(MemberProfile member) {
            NeedRefresh = true;
            MemberProfile = member;
            OnLoad();
        }

    

        private void OnLoad() {            
            IsVisible = (authorizator != null && authorizator.SignedMemberProfile != null);            
            if(NeedRefresh && IsVisible)
                Load();
            MemberDetailViewModel.Close();
        }

        //private System.DateTime lastRefreshDate;
        
        //public override bool NeedRefresh {
        //    get {
                
        //        return System.DateTime.Now.Subtract(lastRefreshDate).TotalSeconds > 10;
        //    }
        //    set {
        //        if (value == false)
        //            lastRefreshDate = System.DateTime.Now;
        //        base.NeedRefresh = value;
        //    }
        //}

        public ICollection<ProjectEvent> ProjectEvents {
            get {
                return projectEvents;
            }
            set {            
                projectEvents = value;                
                OnPropertyChanged("ProjectEvents");
            }
        }

      

      

        private void Load() {

            LoadMemberTasks();
            if(IsTeamTasksVisible)
                LoadMemberTeamTasks();
            LoadMemberEngagedProjects();

            // not using this anymore
            //LoadMemberWorkingWithMembers();

            LoadEvents();

            NeedRefresh = false;
        }

        

        private void LoadEvents() {
            IsLoadingData = true;

            executor.StartBackgroundTask<ICollection<ProjectEvent>>(
                () => { return projectsService.GetUserProjectEvents(); },
                events => {
                    ProjectEvents = new ObservableCollection<ProjectEvent>(events.OrderBy(t => t.When).Take(5));
                    
                    IsLoadingData = false;
                });
        }


        public string PanelName {
            get { return Properties.Resources.My_day; }
        }

        public int PanelDisplayOrder {
            get { return 0; }
        }

        private bool isVisible;
        public bool IsVisible {
            get {
                return isVisible;
            }
            set {
                isVisible = value;
                OnPropertyChanged("IsVisible");
            }
        }

        public bool IsEspecialTab {
            get {
                return true;
            }
        }
      

        [Import(typeof(MyDay))]
        public IView View { get; set; }


        public ICommand RefreshCommand { get; set; }
        public ICommand OnLoadCommand { get; set; }
        public ICommand ShowMemberDetailCommand { get; set; }


        public PanelPlacements PanelPlacement {
            get { return PanelPlacements.Normal; }
        }

        public string ImageUrl {
            get { return "\\Images\\TopMenu\\home.png"; }
        }
    }
}
