﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows.Input;
using ScrumFactory.Composition;
using ScrumFactory.Composition.View;
using ScrumFactory.Composition.ViewModel;

namespace ScrumFactory.Windows.ViewModel {

    [Export]
    [Export(typeof(ITopMenuViewModel))]
    [Export(typeof(IProjectContainer))]
    public class ProjectDetailViewModel : BasePanelViewModel, ITopMenuViewModel, IProjectContainer, INotifyPropertyChanged {

        private IEventAggregator aggregator;
        
        private List<IProjectTabViewModel> projectPanels;

        [Import]
        private IDialogService dialogs { get; set; }

        [ImportingConstructor]
        public ProjectDetailViewModel([Import] IEventAggregator eventAggregator) {

            this.aggregator = eventAggregator;
          

            aggregator.Subscribe<Project>(ScrumFactoryEvent.ViewProjectDetails, ViewProjectDetails);

            aggregator.Subscribe<IProjectTabViewModel>(ScrumFactoryEvent.ShowProjectTab,
             w => {
                 if (w == SelectedProjectTab)
                     return;
                 CloseAllChildWindows();
                 SelectedProjectTab = w;
             });

            aggregator.Subscribe<MemberProfile>(ScrumFactoryEvent.SignedMemberChanged, m => {
                SelectedProjectTab = ProjectPanels.FirstOrDefault();
            });
        }

        private void ViewProjectDetails(Project p) {

            CloseAllChildWindows();

            if (!View.IsVisible)
                dialogs.SelectTopMenu(this);

           

        }

        public string PanelName {
            get { return null; }
        }

        public int PanelDisplayOrder {
            get { return int.MaxValue; }
        }

        public PanelPlacements PanelPlacement {
            get { return PanelPlacements.Project; }
        }

        public string ImageUrl {
            get { return null; }
        }

        [Import(typeof(ProjectDetail))]
        public IView View {
            get;
            set;
        }

        private IProjectTabViewModel selectedProjectTab;
        public IProjectTabViewModel SelectedProjectTab {
            get {
                return selectedProjectTab;
            }
            set {
                selectedProjectTab = value;
                OnPropertyChanged("SelectedProjectTab");
            }
        }

        [ImportMany(typeof(IProjectTabViewModel))]
        public IEnumerable<IProjectTabViewModel> ProjectPanels {
            get {
                return this.projectPanels;
            }
            set {
                projectPanels = value.OrderBy(p => p.PanelDisplayOrder).ToList();
                OnPropertyChanged("ProjectPanels");
            }
        }

        private void CloseAllChildWindows() {        
            OpenedWindows.Clear();
        }
    }
}
